var Steper = window.Steper = function (items) {
  if (!Array.isArray(items)) throw new Error('items must be an array')
  this.items = items
  this.total = items.length
  this.currentPos = 0
  this.nextFns = []
  this.prevFns = []
}

Steper.prototype.next = function (skipCallback) {
  var previous = this.currentPos
  this.currentPos = ++this.currentPos
  if (this.currentPos >= this.total) this.currentPos = 0

  if (skipCallback !== true) this.nextFns.forEach(function (fn) {
    fn.call(this, this.items[this.currentPos], this.items[previous])
  }.bind(this))
}

Steper.prototype.prev = function (skipCallback) {
  var previous = this.currentPos
  this.currentPos = --this.currentPos
  if (this.currentPos < 0) this.currentPos = this.total - 1

  if (skipCallback !== true) this.prevFns.forEach(function (fn) {
    fn.call(this, this.items[this.currentPos], this.items[previous])
  }.bind(this))
}

Steper.prototype.goTo = function(pos) {
  var initialPos = this.currentPos
  var initialItem = this.getCurrent()
  var items = []

  // clamp pos
  if (pos < 0) pos = 0
  if (pos >= this.total) pos = this.total - 1

  if (pos === initialPos) return
  if (pos > initialPos) {
    // forward
    do {
      this.next(true)
      items.push(this.getCurrent())
    } while (this.currentPos < pos);
  } else {
    // backward
    do {
      this.prev(true)
      items.push(this.getCurrent())
    } while (this.currentPos > pos);
  }
  return items;
}

Steper.prototype.getCurrent = function () {
  return this.items[this.currentPos]
}

Steper.prototype.getCurrentPos = function() {
  return this.currentPos;
};

Steper.prototype.onNext = function (fn) {
  this.nextFns.push(fn)
}

Steper.prototype.onPrev = function (fn) {
  this.prevFns.push(fn)
}
